﻿using System;
using System.Collections.Generic;
using System.Configuration;
using System.Linq;
using System.Text;
using System.Threading;

namespace CRMRates
{
    class Program
    {
        private static bool _verbose;
        private static bool _quiet;
        private static bool _multiThreading;
        private static readonly Mutex mutex = new Mutex(true, "{ABF85420-1261-46B6-A1B0-9D839461B9AE}");

        static void Main(string[] args)
        {
            Version thisVersion = System.Reflection.Assembly.GetExecutingAssembly().GetName().Version;
            ConsoleColor oldColor = Console.ForegroundColor;
            Console.ForegroundColor = ConsoleColor.White;
            const string clientName = "Currency Rates Provider for Microsoft Dynamics CRM 2011";
            Console.WriteLine(clientName);
            Console.WriteLine("©2011. Dmitry Kirsanov - http://kirsanov.net");
            Console.WriteLine("Using currency rates xml web service by Mondor Software - http://www.mondor.org");
            Console.ForegroundColor = ConsoleColor.Cyan;
            Console.WriteLine("version {0}.{1}", thisVersion.Major, thisVersion.Minor);
            Console.ForegroundColor = ConsoleColor.White;
            Console.WriteLine(new string('*', clientName.Length));
            _verbose = args.Contains("/v");
            _quiet = args.Contains("/s"); //Either Silent or Service
            if (args.Contains("/l"))
            {
                EventLogger.PrepareEventLog();
                Console.WriteLine("Event Log prepared OK.");
                Console.ForegroundColor = oldColor;
                return;
            }
            if (args.Contains("/?"))
            {
                //Print help
                Console.WriteLine(Strings.ParamList);
                Console.ForegroundColor = oldColor;
                return;
            }
            //Get Mondor's license key
            
            string myKey = ConfigurationManager.AppSettings["MondorKey"];
            if (myKey.Length == 0)
            {
                PrintConsole(
                        "Please specify the key for Mondor.org currency rates web service. It can be acquired at http://www.mondor.org",
                        MessageType.Error, true);
                return;
            }
            //Connect to the database
            using (MSCRMEntities crmDB = new MSCRMEntities())
            {
                TransactionCurrencyBase defaultCurrency =
                    (from x in crmDB.TransactionCurrencyBases where x.ExchangeRate == 1 select x).FirstOrDefault();
                if (defaultCurrency == null)
                {
                    PrintConsole(
                        "Could not find default currency. Please ensure you have a currency with Exchange Rate of 1 in TransactionCurrencyBases table.",
                        MessageType.Error, true);
                    return;
                }
                var curs = from x in crmDB.TransactionCurrencyBases where x.ExchangeRate != 1 select x;
                //Connect to the web service
                using (RateService.rates crates = new RateService.rates())
                {
                    //Ping service
                    int balance = crates.CheckBalance(myKey);
                    DateTime dtExpires = crates.CheckExpirationDate(myKey);
                    PrintConsole(string.Format("Current key: expires at {0}, {1} calls left",
                                               dtExpires.ToShortDateString(), balance));
                    if (dtExpires.Year == 1)
                    {
                        if (balance < 1)
                        {
                            PrintConsole(
                                "It appears like your calls-based key has expired. Replace or recharge your key and try again.",
                                MessageType.Error, true);
                            return;
                        }
                    }
                    else
                    {
                        int daysExpires = (dtExpires - DateTime.Now).Days;
                        if (daysExpires < 14 && daysExpires > 0)
                        {
                            PrintConsole(
                                string.Format(
                                    "Warning! Your key is about to expire in {0} days, at {1}. Make sure you replace or recharge it in time!",
                                    daysExpires, dtExpires.ToLongDateString()), MessageType.Warning, true);
                        }
                        if (daysExpires < 0)
                        {
                            PrintConsole(
                                string.Format ("Your key has expired at {0}. Please acquire replacement before continue. It can be acquired at http://www.mondor.org", dtExpires.ToLongDateString()),
                                MessageType.Error, true);
                            return;
                        }
                    }

                    foreach (TransactionCurrencyBase cur in curs)
                    {
                        decimal newRate = Convert.ToDecimal (crates.Convert(defaultCurrency.ISOCurrencyCode, cur.ISOCurrencyCode, 1, myKey));
                        if (newRate == 0)
                        {
                            PrintConsole(
                                string.Format(
                                    "Error: got currency rate 0 (zero) when checking for currency {0}. Please try again later or verify if your key balance is depleted.",
                                    cur.ISOCurrencyCode), MessageType.Error);
                            return;
                        }
                        cur.ExchangeRate = Decimal.Round (newRate, 4);
                        PrintConsole(string.Format("Currency rate for {0} set to {1}", cur.ISOCurrencyCode, cur.ExchangeRate));
                    }
                    //Update database
                    crmDB.SaveChanges();
                    PrintConsole("All changes saved to the database ok.");
                }
            }
        }

        public static void PrintConsole(string message, MessageType msgType = MessageType.Info,
                                        bool enforce = false, bool overwrite = false)
        {
            //Print time
            if (_quiet)
            {
                LogEvent(message, msgType, enforce);
                return;
            }
            if (!_verbose & !enforce & msgType == MessageType.Info) return;

            //Simple line break?
            if (msgType == MessageType.LineBreak)
            {
                Console.Write("\n");
                return;
            }

            ConsoleColor oldColor = Console.ForegroundColor;
            Console.ForegroundColor = ConsoleColor.White;
            if (!overwrite)
            {
                Console.Write(string.Format("[{0} {1}]: ", DateTime.Now.ToShortDateString(),
                                            DateTime.Now.ToShortTimeString()));
            }
            //Print the rest
            switch (msgType)
            {
                case MessageType.Info:
                    if (!_verbose & !enforce) return;
                    Console.ForegroundColor = ConsoleColor.White;
                    break;
                case MessageType.Warning:
                    Console.ForegroundColor = ConsoleColor.Yellow;
                    break;
                case MessageType.Error:
                    Console.ForegroundColor = ConsoleColor.Red;
                    break;
            }
            if (!overwrite) Console.WriteLine(message);
            else
            {
                Console.Write(new string('\b', message.Length));
                Console.Write(message);
            }
            Console.ForegroundColor = oldColor;
        }

        private static void LogEvent(string message, MessageType msgType = MessageType.Info,
                                     bool enforce = false)
        {
            if (!_verbose & !enforce & msgType == MessageType.Info) return;
            switch (msgType)
            {
                case MessageType.Info:
                    if (!_verbose & !enforce) return;
                    EventLogger.WriteEventLog(message, System.Diagnostics.EventLogEntryType.Information, 0);
                    break;
                case MessageType.Warning:
                    EventLogger.WriteEventLog(message, System.Diagnostics.EventLogEntryType.Warning, 1);
                    break;
                case MessageType.Error:
                    EventLogger.WriteEventLog(message, System.Diagnostics.EventLogEntryType.Error, 5);
                    break;
            }
        }

    } //end class Program
    public enum MessageType
    {
        Info,
        Warning,
        Error,
        LineBreak
    }
}
